/* ***********************************************************
 * This file was automatically generated on 2012-10-12.      *
 *                                                           *
 * If you have a bugfix for this file and want to commit it, *
 * please fix the bug in the generator. You can find a link  *
 * to the generator git on tinkerforge.com                   *
 *************************************************************/

#ifndef BRICKLET_INDUSTRIAL_QUAD_RELAY_H
#define BRICKLET_INDUSTRIAL_QUAD_RELAY_H

#include "ip_connection.h"

/**
 * \defgroup BrickletIndustrialQuadRelay IndustrialQuadRelay Bricklet
 */

/**
 * \ingroup BrickletIndustrialQuadRelay
 *
 * Device for controlling up to 4 Solid State Relays
 */
typedef Device IndustrialQuadRelay;

/**
 * \ingroup BrickletIndustrialQuadRelay
 *
 * This callback is triggered whenever a monoflop timer reaches 0. The
 * parameters contain the involved pins and the current value of the pins
 * (the value after the monoflop).
 */
#define INDUSTRIAL_QUAD_RELAY_CALLBACK_MONOFLOP_DONE 8

/**
 * \ingroup BrickletIndustrialQuadRelay
 *
 * Creates an object with the unique device ID \c uid. This object can then be
 * added to the IP connection.
 */
void industrial_quad_relay_create(IndustrialQuadRelay *industrial_quad_relay, const char *uid);

/**
 * \ingroup BrickletIndustrialQuadRelay
 *
 * Sets the output value with a bitmask. The bitmask
 * is 16 bit long, *true* refers to a closed relay and *false* refers to 
 * an open relay.
 * 
 * For example: The value 0b0000000000000011 will close the relay 
 * of pins 0-1 and open the other pins.
 * 
 * If no groups are used (see {@link industrial_quad_relay_set_group}), the pins correspond to the
 * markings on the Quad Relay Bricklet.
 * 
 * If groups are used, the pins correspond to the element in the group.
 * Element 1 in the group will get pins 0-3, element 2 pins 4-7, element 3
 * pins 8-11 and element 4 pins 12-15.
 */
int industrial_quad_relay_set_value(IndustrialQuadRelay *industrial_quad_relay, uint16_t value_mask);

/**
 * \ingroup BrickletIndustrialQuadRelay
 *
 * Returns the bitmask as set by {@link industrial_quad_relay_set_value}.
 */
int industrial_quad_relay_get_value(IndustrialQuadRelay *industrial_quad_relay, uint16_t *ret_value_mask);

/**
 * \ingroup BrickletIndustrialQuadRelay
 *
 * Configures a monoflop of the pins specified by the first parameter
 * bitmask.
 * 
 * The second parameter is a bitmask with the desired value of the specified
 * pins (*true* means relay closed and *false* means relay open).
 * 
 * The third parameter indicates the time (in ms) that the pins should hold
 * the value.
 * 
 * If this function is called with the parameters 
 * ((1 << 0) | (1 << 3), (1 << 0), 1500):
 * Pin 0 will close and pin 3 will open. In 1.5s pin 0 will open and pin
 * 3 will close again.
 * 
 * A monoflop can be used as a fail-safe mechanism. For example: Lets assume you
 * have a RS485 bus and a Quad Relay Bricklet connected to one of the slave
 * stacks. You can now call this function every second, with a time parameter
 * of two seconds and pin 0 closed. Pin 0 will be closed all the time. If now
 * the RS485 connection is lost, then pin 0 will be opened in at most two seconds.
 */
int industrial_quad_relay_set_monoflop(IndustrialQuadRelay *industrial_quad_relay, uint16_t pin_mask, uint16_t value_mask, uint32_t time);

/**
 * \ingroup BrickletIndustrialQuadRelay
 *
 * Returns (for the given pin) the current value and the time as set by
 * {@link industrial_quad_relay_set_monoflop} as well as the remaining time until the value flips.
 * 
 * If the timer is not running currently, the remaining time will be returned
 * as 0.
 */
int industrial_quad_relay_get_monoflop(IndustrialQuadRelay *industrial_quad_relay, uint8_t pin, uint16_t *ret_value, uint32_t *ret_time, uint32_t *ret_time_remaining);

/**
 * \ingroup BrickletIndustrialQuadRelay
 *
 * Sets a group of Quad Relay Bricklets that should work together. You can
 * find Bricklets that can be grouped together with {@link industrial_quad_relay_get_available_for_group}.
 * 
 * The group consists of 4 elements. Element 1 in the group will get pins 0-3,
 * element 2 pins 4-7, element 3 pins 8-11 and element 4 pins 12-15.
 * 
 * Each element can either be one of the ports ('a' to 'd') or 'n' if it should
 * not be used.
 * 
 * For example: If you have two Quad Relay Bricklets connected to port A and
 * port B respectively, you could call with "['a', 'b', 'n', 'n']".
 * 
 * Now the pins on the Quad Relay on port A are assigned to 0-3 and the
 * pins on the Quad Relay on port B are assigned to 4-7. It is now possible
 * to call {@link industrial_quad_relay_set_value} and control two Bricklets at the same time.
 */
int industrial_quad_relay_set_group(IndustrialQuadRelay *industrial_quad_relay, char group[4]);

/**
 * \ingroup BrickletIndustrialQuadRelay
 *
 * Returns the group as set by {@link industrial_quad_relay_set_group}
 */
int industrial_quad_relay_get_group(IndustrialQuadRelay *industrial_quad_relay, char ret_group[4]);

/**
 * \ingroup BrickletIndustrialQuadRelay
 *
 * Returns a bitmask of ports that are available for grouping. For example the
 * value 0b0101 means: Port *A* and Port *C* are connected to Bricklets that
 * can be grouped together.
 */
int industrial_quad_relay_get_available_for_group(IndustrialQuadRelay *industrial_quad_relay, uint8_t *ret_available);

/**
 * \ingroup BrickletIndustrialQuadRelay
 *
 * Returns the name (including the hardware version), the firmware version
 * and the binding version of the device. The firmware and binding versions are
 * given in arrays of size 3 with the syntax [major, minor, revision].
 */
int industrial_quad_relay_get_version(IndustrialQuadRelay *industrial_quad_relay, char ret_name[40], uint8_t ret_firmware_version[3], uint8_t ret_binding_version[3]);

/**
 * \ingroup BrickletIndustrialQuadRelay
 *
 * Registers a callback with ID \c id to the function \c callback.
 */
void industrial_quad_relay_register_callback(IndustrialQuadRelay *industrial_quad_relay, uint8_t id, void *callback);

#endif
